"""
Version information for Ranger, created during installation.

Do not add this file to the repository.

"""

import datetime

version = '0.1.dev_20140719184002'
date = 'Sat Jul 19 18:40:02 2014'

# Was Ranger built from a development version? If so, remember that the major
# and minor versions reference the "target" (rather than "current") release.
dev = True

# Format: (name, major, min, revision)
version_info = ('Ranger', '0', '1', None)

# Format: a 'datetime.datetime' instance
date_info = datetime.datetime(2014, 7, 19, 18, 40, 2, 314838)

# Format: (vcs, vcs_tuple)
vcs_info = (None, (None, None))

